
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "board.h"
#include "agent.h"

int BOARD_SIZE=0;

int numGhosts;

char ** TheBoard;
char ** OrigBoard;
int maxScore=0;


char charSet[6] = {'0', '*', '.', '/', '\\', ' '};


Agent Ghosts[MAX_GHOSTS];
Agent Pacman;


// initBoard allocates memory for the board
void initBoard(void) {
	int i;

	// The board is just a sequence of strings. 
	// This declares a pointer to each string, but no space for the characters
	// We declare two copies, one for actual use, and one that just is the original board specification

	TheBoard = (char **) calloc(BOARD_SIZE, sizeof(char *));
	OrigBoard = (char **) calloc(BOARD_SIZE, sizeof(char *));

	for (i=0; i<BOARD_SIZE; i++) {
		// This now declares space for each character
		// Width of the board plus one for a trailing 0
		TheBoard[i] = (char *) calloc(BOARD_SIZE+1, sizeof(char));
		OrigBoard[i] = (char *) calloc(BOARD_SIZE+1, sizeof(char));
	}
}


// deInitBoard deallocates memory for the board
void deInitBoard(void) {
	int i;

	for (i=0; i<BOARD_SIZE; i++) {
		// This now declares space for each character
		// Width of the board plus one for a trailing 0
		free(TheBoard[i]);
		free(OrigBoard[i]);
	}

	free(TheBoard);
	free(OrigBoard);
	
	
}

int loadBoardFromFile(char * fileName) {
	
	FILE * inputFile;
	char temp[256];

	int i, j; // Loop counters
	char * c; // Board grid value
	int v=0;
	int g; // Loop counter for loading ghosts

	// Open file
	inputFile = fopen(fileName, "r");

	if (inputFile==NULL) {
		fprintf(stderr, "Unable to open board file: %s. Aborting.\n", fileName);
		exit(1);
	}

	fgets(temp, 255, inputFile);
	if (sscanf(temp, "%d\n", &BOARD_SIZE)!=1) {
		fprintf(stderr, "Error reading from board file. Aborting.\n");
		exit(1);
	}
	printf("Loading board of size: %d\n", BOARD_SIZE);

	initBoard();

	for (i=0; i<BOARD_SIZE; i++) {
		// Read a line from the file
		fgets(temp, 255, inputFile);

		// Remove the leading zero
		c=strtok(temp, ",");
		for (j=0; j<BOARD_SIZE; j++) {
			// Get the next character
			c=strtok(NULL, ",");
			if (c!=NULL) {
				// Convert character to decimal
				if (sscanf(c, "%d", &v)!=1) {
					fprintf(stderr, "Error reading from board file. Aborting.\n");
					exit(1);
				} else {
					// Look up a value from the character set and insert into OrigBoard
					OrigBoard[i][j] = charSet[v];
					if (OrigBoard[i][j]=='.') maxScore++;
				}
			}
		}
	}
	
	// Read pacman from the file
	NewAgent(&Pacman);
	fgets(temp, 255, inputFile);
	sscanf(temp, "(%d, %d, %d)", &Pacman.xPos, &Pacman.yPos, &Pacman.symbol);

	printf("Pacman @: %d, %d, %d\n", Pacman.xPos, Pacman.yPos, Pacman.symbol);
	

	// Read a line from the file
	fgets(temp, 255, inputFile);
	sscanf(temp, "%d\n", &numGhosts);

	// Load each ghost
	for (g=0; g<numGhosts; g++) {
		// Read in ghosts from file
		NewAgent(&Ghosts[g]);
		fgets(temp, 255, inputFile);
		sscanf(temp, "<%d, %d, %d>", &Ghosts[g].xPos, &Ghosts[g].yPos, &Ghosts[g].symbol);
		printf("Ghost @: %d, %d, %d\n", Ghosts[g].xPos, Ghosts[g].yPos, Ghosts[g].symbol);
	}
	
	// Copy the board to the actual one
	copyBoard(OrigBoard, TheBoard);

	// Position the agents on the board
	TheBoard[Pacman.yPos][Pacman.xPos]=Pacman.symbol;
	for (g=0; g<numGhosts; g++) {
		TheBoard[Ghosts[g].yPos][Ghosts[g].xPos]=Ghosts[g].symbol;
	}

	return 1;
}


// Copies one board over another
void copyBoard(char ** from, char ** to) {
	int i;
	for (i=0; i<BOARD_SIZE; i++) {
		strcpy(to[i], from[i]);
	}
}

// Clears the screen, and then displays a board
void drawBoard(char ** board) {
	int i;

	system("cls");
	for (i=0; i<BOARD_SIZE; i++) {
		printf("%s\n", board[i]);
	}
}


void modifyBoards(int x, int y, char c) {
	//TheBoard[y][x] = c;
	OrigBoard[y][x] = c;
}

// Displays an agents pending moves to the user - on the board.
int dumpMoveListBoard(Agent * a) {
	int i, j=0, x, y;
	x = a->xPos;
	y = a->yPos;

	for (i=0; i < a->numMovesInList; i++) {
		switch (a->moveList[i]) {
		case UP:
			y--;
			break;

		case DOWN:
			y++;
			break;

		case LEFT:
			x--;
			break;

		case RIGHT:
			x++;
			break;
		}
		TheBoard[y][x] = j+'0';
		j++;
		j%=10;
	}
	drawBoard(TheBoard);
	return 0;
}


// Handles moving an agent
int moveAgent(Agent * a) {
	int oldX, oldY;
	int moved = FALSE;
	
	// Record where the agent was
	oldX = a->xPos;
	oldY = a->yPos;

	// Is this agent trying to move?
	if (a->numMovesInList) {
		// Check that the desired location is free
		switch (a->moveList[0]) {
		
		case UP:
			if (a->yPos==0) {
				if (FREE(a->xPos,BOARD_SIZE-1)) {
					a->yPos = BOARD_SIZE-1;
					moved = TRUE;
				}
			} else {
				if (FREE(a->xPos,a->yPos-1)) {
					a->yPos = a->yPos-1;
					moved = TRUE;
				}
			}
			break;

		case DOWN:
			if (a->yPos==BOARD_SIZE-1) {
				if (FREE(a->xPos,0)) {
					a->yPos = 0;
					moved = TRUE;
				}
			} else {
				if (FREE(a->xPos,a->yPos+1)) {
					a->yPos = a->yPos+1;
					moved = TRUE;
				}
			}
			break;

		case LEFT:
			if (a->xPos==0) {
				if (FREE(BOARD_SIZE-1,a->yPos)) {
					a->xPos = BOARD_SIZE-1;
					moved = TRUE;
				}
			} else {
				if (FREE(a->xPos-1,a->yPos)) {
					a->xPos = a->xPos-1;
					moved = TRUE;
				}
			}
			break;

		case RIGHT:
			if (a->xPos==BOARD_SIZE-1) {
				if (FREE(0,a->yPos)) {
					a->xPos = 0;
					moved = TRUE;
				}
			} else {
				if (FREE(a->xPos+1,a->yPos)) {
					a->xPos = a->xPos+1;
					moved = TRUE;
				}
			}
			break;
		}
		if (moved==TRUE) {
			RemoveMove(a, 0);
			TheBoard[a->yPos][a->xPos] = a->symbol;
			TheBoard[oldY][oldX] = OrigBoard[oldY][oldX];
		}
	}

	return 1;
}
